/*RC522 MODULE    Uno/Nano     MEGA
  IRQ             N/A          N/A  BR
  NSS             D10          D53  WH/BR
  SCK             D13          D52  GR
  MOSI            D11          D51  WH/GR
  MISO            D12          D50  OR
  GND             GND          GND  WH/OR
  RST             D9           D5   BL
  3.3V            3.3V         3.3V WH/BL

  WEMOS
  IRQ             N/A WH/BL (Blue)
  NSS (SDA)       D8  WH/GR (Black)
  SCK             D5  GR  (Yellow)
  MOSI            D7  WH/OR (Red)
  MISO            D6  BLUE (Green)
  GND             GND OR   (Orange)
  RST             D0  WH/BR (Brown)
  3.3V            VCC BR  (Grey)

  H0 Blue  { 53, 5},
  H1 Pink  { 49, 6},
  H2 Silver  { 48, 7 },
  H3 Red    {47, 8}
  Test ok with Mega
  Tested OK with 6m of cat5

  //Normal run
  //Vcc 4.85v
  //D4 3.36V
  //D3 3.36v
  //D2 84mv
  //D1 84mV
  //RX 3.36V
  //TX 3.36V
  //Res 3.36V
  //A0 60mV
  //D0 84mV
  //D5 84mV
  //D6 84mV
  //D7 85mV
  //D8 85mV
  //3.3v 3.36V


  //Pins good for input D0,D1,D2,D5,D6,D7
  //D0 GPIO16 dont care

  //D1 GPIO05 Dont Care

  //D2 GPIO04 Dont Care

  //D3 GPIO00 can not be low during a reset, but must be low during a reflash
  //D3 GPIO00 can be high during a reset but must be low during a reflash
  //D3 GPIO00 can be used with wemos relay shield
  //D3 pull up

  //D4 GPIO02 can not be low during a reset
  //D4 GPIO02 can be high during a reset and reflash
  //D4 GPIO02 cant be used with wemos relay shield. Will Not reset
  //D4 pull up

  //D5 GPIO14 Dont Care
  //D5 GPIO14 SPI Clock

  //D6 GPIO12 Dont Care
  //D6 GPIO12 SPI MISO

  //D7 GPIO13 (Serial.swap() -> RX) Dont Care
  //D7 GPIO13 SPI MOSI

  //D8 GPIO15 (Serial.swap() -> TX) can not be high during a reset or a reflash
  //D8 GPIO15 (Serial.swap() -> TX) can be low during a reset and reflash
  //D8 pull down

  //TX GPIO01

  //RX GPIO03


*/

#include <Arduino.h>

#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>
#include <WebSocketsClient.h>
#include <Hash.h>
#define Serial Serial
#include <PS2X_esp_lib.h>  //for v1.6

/******************************************************************
   set pins connected to PS2 controller:
     - 1e column: original
     - 2e colmun: Stef?
   replace pin numbers by the ones you use
 ******************************************************************/
#define PS2_DAT       D6 //MISO - PS2 Pin 1 Data (Controller -> Playstation)  Requires Pullups 47k 
#define PS2_CMD       D7 //MOSI - PS2 Pin 2 Command (Playstation -> Controller)
#define PS2_SEL       D0 //       PS2 Pin 6 Attention
#define PS2_CLK       D5 //CLK  - PS2 Pin 7 Clock

//PS2 Pin 3 - Vibratiion Motor 7.2 - 9v?
//PS2 Pin 4 - Ground
//PS2 Pin 5 - 3.3v
//PS2 Pin 8 - NC
//PS2 Pin 9 - ACK

/******************************************************************
   select modes of PS2 controller:
     - pressures = analog reading of push-butttons
     - rumble    = motor rumbling
   uncomment 1 of the lines for each mode selection
 ******************************************************************/
#define pressures   true
//#define pressures   false
#define rumble      true
//#define rumble      false

PS2X ps2x; // create PS2 Controller Class
ESP8266WiFiMulti WiFiMulti;
WebSocketsClient webSocket;

//Flow Control variables
unsigned long nextMillis = 0;

//Controller Variables
unsigned long nextLEDMillis = 0;
int error = 0;
byte type = 0;
byte vibrate = 0;
int count = 0;
unsigned char pGamePadOld[21]; //Stores the previous GamePad data for comparison against the current Gamepad data
unsigned char* pGamePad;  //A pointer to the data

void webSocketEvent(WStype_t type, uint8_t * payload, size_t lenght)
{
  switch (type)
  {
    case WStype_DISCONNECTED:
      Serial.printf("[WSc] Disconnected!\n");
      break;
    case WStype_CONNECTED:
      {
        Serial.printf("[WSc] Connected to url: %s\n",  payload);

        // send message to server when Connected
        webSocket.sendTXT("Connected");
      }
      break;
    case WStype_TEXT:
      //Serial.printf("[WSc] get text: %s\n", payload);
      //Any message sent from the server to this client resets the LED status
      //The LED provides quick status of the comms link. If flashing ~0.5Hz then comms ok
      nextLEDMillis = millis() + 1000;
      digitalWrite(D1, 1);
      // send message to server
      // webSocket.sendTXT("message here");
      break;
    case WStype_BIN:
      Serial.printf("[WSc] get binary lenght: %u\n", lenght);
      hexdump(payload, lenght);

      // send data to server
      // webSocket.sendBIN(payload, lenght);
      break;
  }
}

void setup()
{
  // Serial stuff
  Serial.begin(500000);
//  Serial.setDebugOutput(true);
  Serial.println();
  Serial.println();
  Serial.println();
  for (uint8_t t = 1; t > 0; t--) {
    Serial.printf("[SETUP] BOOT WAIT %d...\n", t);
    Serial.flush();
    delay(1000);
  }

  // Wifi Stuff
  WiFi.mode(WIFI_STA);
  WiFiMulti.addAP("AP", "min8char"); //AP name is "AP" password "min8char" password can be blank but if not must be at least 8 chars
  while (WiFiMulti.run() != WL_CONNECTED) {
    delay(100);
  }
  webSocket.begin("192.168.4.1", 81); //It assumes the server will be this IP. BY default it is
  webSocket.onEvent(webSocketEvent);

  //PS2 Controller Stuff
  error = 1;
  while (error == 1)
  {
    delay(300);  //added delay to give wireless ps2 module some time to startup, before configuring it
    //CHANGES for v1.6 HERE!!! **************PAY ATTENTION*************

    //setup pins and settings: GamePad(clock, command, attention, data, Pressures?, Rumble?) check for error
    error = ps2x.config_gamepad(PS2_CLK, PS2_CMD, PS2_SEL, PS2_DAT, pressures, rumble);

    if (error == 0) {
      Serial.print("Found Controller, configured successful ");
      Serial.print("pressures = ");
      if (pressures)
        Serial.println("true ");
      else
        Serial.println("false");
      Serial.print("rumble = ");
      if (rumble)
        Serial.println("true)");
      else
        Serial.println("false");
      Serial.println("Try out all the buttons, X will vibrate the controller, faster as you press harder;");
      Serial.println("holding L1 or R1 will print out the analog stick values.");
      Serial.println("Note: Go to www.billporter.info for updates and to report bugs.");
    }
    else if (error == 1)
      Serial.println("No controller found, check wiring, see readme.txt to enable debug. visit www.billporter.info for troubleshooting tips");

    else if (error == 2)
      Serial.println("Controller found but not accepting commands. see readme.txt to enable debug. Visit www.billporter.info for troubleshooting tips");

    else if (error == 3)
      Serial.println("Controller refusing to enter Pressures mode, may not support it. ");

    //  Serial.print(ps2x.Analog(1), HEX);

    type = ps2x.readType();
    switch (type) {
      case 0:
        Serial.println("Unknown Controller type found ");
        break;
      case 1:
        Serial.println("DualShock Controller found ");
        break;
      case 2:
        Serial.println("GuitarHero Controller found ");
        break;
      case 3:
        Serial.println("Wireless Sony DualShock Controller found ");
        break;
    }
  }

  pinMode(D1, OUTPUT);
  digitalWrite(D1, 0);
  ps2x.read_gamepad(false, vibrate); //read controller and set large motor to spin at 'vibrate' speed
  delay(1000);
  pGamePad = ps2x.get_gamepad();
  memcmp(pGamePad, pGamePadOld, 21);
}

void loop()
{
  webSocket.loop();

  if (error == 1) //skip loop if no controller found
    return;

  if (type == 1)
  {
    ps2x.read_gamepad(false, vibrate); //read controller and set large motor to spin at 'vibrate' speed
    vibrate = ps2x.Analog(PSAB_CROSS);

    pGamePad = ps2x.get_gamepad();
    if (memcmp(pGamePad, pGamePadOld, 21))
    {
      webSocket.sendTXT(pGamePad, 21);
      memcpy( pGamePadOld, pGamePad, 21);
      //Uncomment the following for debug if you need to see the PS2x Gampad values.
      //Or if connecting directly to PC and you are simply using the ESP8266 to provide
      //way to connect the PS2x to the PC.
      
//      for (int i = 0; i < 21; i++) {
//        if(GamePadData[i] < 0x10) {Serial.print("0");}
//        Serial.print(pGamePad[i], HEX);
//        Serial.print("::");
//      }
//      Serial.println();
    }
  }
  if (digitalRead(D1) && ( millis() > nextLEDMillis ))
  {
    //If timeout turn of comms status LED.
    digitalWrite(D1, 0);
  }
}
